@@ -14,6 +14,8 @@
 /* Accelerometer support taken from http://wiki.maemo.org/Accelerometers */
 
 static int ocnt = 0;
+
+#ifdef N900
 static int oax = 0;
 static int oay = 0;
 static int oaz = 0;
@@ -46,5 +48,12 @@ int liqaccel_read(int *ax, int *ay, int *az) {
     oay = *ay;
     oaz = *az;
     ocnt++;
+    return 0; 
+}
+
+#else
+
+int liqaccel_read(int *ax, int *ay, int *az) {
     return 0;
 }
+#endif
