@@ -55,21 +55,40 @@ void loadTileProviders() {
     g_key_file_free(keyfile);
 }
 
+void* getProviderByName(char* name, GList* allProviders) {
+	if (name != NULL) {
+		GList* elem = allProviders;
+
+		while (elem != NULL) {
+			BackgroundQueryProvider* provider = elem -> data;
+
+			if (g_str_equal(provider -> name, name)) {
+    			return provider;
+    		}
+			elem = elem -> next;
+		}
+    }
+	return allProviders -> data;
+}
+
 void loadConfig() {
     loadTileProviders();
 	initGeocodingProviders();
+	initRoutingProviders();
 
     GKeyFile *keyfile;
     GError *error = NULL;
     keyfile = g_key_file_new();
     gchar *tileProvider = NULL;
     gchar *geocodingProvider = NULL;
+    gchar *routingProvider = NULL;
     gchar *ttfFont = NULL;
 
     if (g_key_file_load_from_file(keyfile, GLOBAL_CONFIG_FILE, 0, &error)) {
         options.referer = g_key_file_get_string(keyfile, "Network", "httpReferer", NULL);
         options.userAgent = g_key_file_get_string(keyfile, "Network", "httpUserAgent", NULL);
         options.curlVerbose = g_key_file_get_integer(keyfile, "Network", "curlVerbose", NULL);
+        options.forceIPv4 = g_key_file_get_integer(keyfile, "Network", "forceIPv4", NULL);
         canvas.zoom = g_key_file_get_integer(keyfile, GROUP_DEFAULTS, "zoom", NULL);
         canvas.tilex = g_key_file_get_integer(keyfile, GROUP_DEFAULTS, "tilex", NULL);
         canvas.tiley = g_key_file_get_integer(keyfile, GROUP_DEFAULTS, "tiley", NULL);
@@ -101,6 +120,14 @@ void loadConfig() {
         options.locallyLoadedFadeTime = 200;
     }
 
+    options.mouseHoldToPopupTime = 1000;
+    options.snapToRouteToleranceMeters = 30;
+    options.outsideRouteToleranceMeters = 50;
+    options.recalculateRouteAfterOutSideRouteSeconds = 10;
+    options.useFading = FALSE;
+    options.useTileSync = FALSE;
+    options.popupEntryMargin = 10;
+
     g_key_file_free(keyfile);
     keyfile = g_key_file_new();
 
@@ -118,6 +145,8 @@ void loadConfig() {
         }
         tileProvider = g_key_file_get_string(keyfile, "Options", "tileProvider", NULL);
         geocodingProvider = g_key_file_get_string(keyfile, "Options", "geocodingProvider", NULL);
+        routingProvider = g_key_file_get_string(keyfile, "Options", "routingProvider", NULL);
+        options.loadLastRoute = g_key_file_get_integer(keyfile, "Options", "loadLastRoute", NULL);
     } else {
         fprintf(stderr, "Error loading %s %s\n", USER_CONFIG_FILE, error->message);
     }
@@ -139,18 +168,8 @@ void loadConfig() {
         canvas.provider = tileProviders -> data;
     }
 
-    if (geocodingProvider != NULL) {
-    	int i;
-    	for (i = 0; i < geocodingProviders -> len; i++) {
-    		if (g_str_equal(g_array_index(geocodingProviders, GeocodingProvider, i).name, geocodingProvider)) {
-    			options.geocodingProvider = g_array_index(geocodingProviders, GeocodingProvider, i);
-    		}
-    	}
-    }
-
-    if (options.geocodingProvider.name == NULL) {
-    	options.geocodingProvider = g_array_index(geocodingProviders, GeocodingProvider, 0);
-    }
+	options.geocodingProvider = getProviderByName(geocodingProvider, geocodingProviders);
+	options.routingProvider = getProviderByName(routingProvider, routingProviders);
 }
 
 void saveConfig() {
@@ -169,7 +188,9 @@ void saveConfig() {
     g_key_file_set_double(keyfile, "CurrentView", "x", canvas.x);
     g_key_file_set_double(keyfile, "CurrentView", "y", canvas.y);
     g_key_file_set_string(keyfile, "Options", "tileProvider", canvas.provider -> name);
-    g_key_file_set_string(keyfile, "Options", "geocodingProvider", options.geocodingProvider.name);
+    g_key_file_set_string(keyfile, "Options", "geocodingProvider", options.geocodingProvider -> name);
+    g_key_file_set_string(keyfile, "Options", "routingProvider", options.routingProvider -> name);
+    g_key_file_set_integer(keyfile, "Options", "loadLastRoute", canvas.route.directions == NULL ? 0 : 1);
     gchar* data = g_key_file_to_data(keyfile, &lenght, &error);
     if (data == NULL || lenght <= 0) {
         fprintf(stderr, "Error dumping setting to data: %s\n", error->message);
@@ -181,10 +202,5 @@ void saveConfig() {
         } else {
             fprintf(stderr, "Unable to open %s for writing.\n", USER_CONFIG_FILE);
         }
-
-        // g_file_set_contents cannot write to files, wtf?
-        //        if (!g_file_set_contents(USER_CONFIG_FILE, data, -1, &error)) {
-        //            fprintf(stderr, "Error writing config to %s: %s\n", USER_CONFIG_FILE, error->message);
-        //        }
     }
 }
