@@ -35,7 +35,7 @@ void addConsoleLine(char* text, int r, int g, int b) {
 	}
 	g_queue_push_tail(consoleLines, line);
 }
-void deleteOldLines() {
+void deleteOldConsoleLines() {
 	if (consoleLines == NULL) {
 		return;
 	}
