@@ -11,7 +11,6 @@
  * Author: Damian Waradzyn
  */
 
-
 #include "cloudmade_geocoder.h"
 
 #define KEY_FEATURES "features"
@@ -24,7 +23,6 @@
 #define KEY_WIKIPEDIA "Wikipedia"
 
 #define CLOUDMADE_QUERY_URL "http://geocoding.cloudmade.com/e4b1777b4b5154d69dbfc4678216183a/geocoding/v2/find.js?query=%s&return_location=true"
-
 char* cloudmade_prepare_url(char* encodedQuery) {
 	char* url;
 	if (asprintf(&url, CLOUDMADE_QUERY_URL, encodedQuery) < 0) {
@@ -47,6 +45,10 @@ void cloudmade_parse_response(char* response) {
 	int i;
 
 	json_object * jobj = json_tokener_parse(response);
+	if (is_error(jobj)) {
+		searchResultsStatus = PARSE_ERROR;
+		return;
+	}
 	json_object * features = json_object_object_get(jobj, KEY_FEATURES);
 	if (features != NULL) {
 		length = json_object_array_length(features);
@@ -57,8 +59,10 @@ void cloudmade_parse_response(char* response) {
 			readCloudmadeWorldCoordinate(json_object_array_get_idx(json_object_object_get(feature, KEY_BOUNDS), 0), &result -> bounds[0]);
 			readCloudmadeWorldCoordinate(json_object_array_get_idx(json_object_object_get(feature, KEY_BOUNDS), 1), &result -> bounds[1]);
 			json_object *properties = json_object_object_get(feature, KEY_PROPERTIES);
-			result -> name = json_object_get_string(json_object_object_get(properties, KEY_NAME));
-			result -> description = json_object_get_string(json_object_object_get(properties, KEY_IS_IN));
+			result -> name = calloc(300, sizeof(char));
+			ascifyAndStripTags((char*) json_object_get_string(json_object_object_get(properties, KEY_NAME)), result -> name);
+			result -> description = calloc(300, sizeof(char));
+			ascifyAndStripTags((char*) json_object_get_string(json_object_object_get(properties, KEY_IS_IN)), result -> description);
 			result -> wikipedia = json_object_get_string(json_object_object_get(properties, KEY_WIKIPEDIA));
 
 			//                    fprintf(stderr, "Name: '%s' Description: '%s', center: %f, %f, Wikipedia: '%s'\n", result -> name, result -> description, result -> centroid.latitude, result -> centroid.longitude, result-> wikipedia == NULL ? "null": result -> wikipedia);
