@@ -20,9 +20,9 @@
 #include "google_maps_geocoder.h"
 #include "geocoder.h"
 
-volatile SearchResultStatus searchResultsStatus = NO_QUERY;
+volatile BackgroundQueryStatus searchResultsStatus = NO_QUERY;
 
-GArray* geocodingProviders;
+GList* geocodingProviders;
 
 GList* geocodingResults;
 
@@ -30,28 +30,30 @@ GList* geocodingResults;
 #define QUERY_TMP_FILENAME "/tmp/cloudgps-query-tmp.js"
 
 void initGeocodingProviders() {
-	geocodingProviders = g_array_new(FALSE, FALSE, sizeof(GeocodingProvider));
+	BackgroundQueryProvider* p;
+	p = calloc(1, sizeof(BackgroundQueryProvider));
+	p -> name = "CloudMade";
+	p -> parseResponse = &cloudmade_parse_response;
+	p -> prepareUrl = &cloudmade_prepare_url;
+	geocodingProviders = g_list_append(geocodingProviders, p);
 
-	GeocodingProvider p;
-	p.name = "CloudMade";
-	p.parseResponse = &cloudmade_parse_response;
-	p.prepareUrl = &cloudmade_prepare_url;
-	geocodingProviders = g_array_append_val(geocodingProviders, p);
+	p = calloc(1, sizeof(BackgroundQueryProvider));
+	p -> name = "Google (address only)";
+	p -> parseResponse = &google_parse_response;
+	p -> prepareUrl = &google_prepare_url;
+	geocodingProviders = g_list_append(geocodingProviders, p);
 
-	p.name = "Google (address only)";
-	p.parseResponse = &google_parse_response;
-	p.prepareUrl = &google_prepare_url;
-	geocodingProviders = g_array_append_val(geocodingProviders, p);
+	p = calloc(1, sizeof(BackgroundQueryProvider));
+	p -> name = "Google Maps";
+	p -> parseResponse = &google_maps_parse_response;
+	p -> prepareUrl = &google_maps_prepare_url;
+	geocodingProviders = g_list_append(geocodingProviders, p);
 
-	p.name = "Google Maps";
-	p.parseResponse = &google_maps_parse_response;
-	p.prepareUrl = &google_maps_prepare_url;
-	geocodingProviders = g_array_append_val(geocodingProviders, p);
-
-	p.name = "Google Maps Local";
-	p.parseResponse = &google_maps_parse_response;
-	p.prepareUrl = &google_maps_local_prepare_url;
-	geocodingProviders = g_array_append_val(geocodingProviders, p);
+	p = calloc(1, sizeof(BackgroundQueryProvider));
+	p -> name = "Google Maps Local";
+	p -> parseResponse = &google_maps_parse_response;
+	p -> prepareUrl = &google_maps_local_prepare_url;
+	geocodingProviders = g_list_append(geocodingProviders, p);
 }
 
 /* to_hex and url_encode taken from: http://www.geekhideout.com/urlcode.shtml */
@@ -87,7 +89,7 @@ int geocodeBgThread(void* query) {
     encoded = url_encode((char*) query);
     free(query);
 
-    url = options.geocodingProvider.prepareUrl(encoded);
+    url = options.geocodingProvider -> prepareUrl(encoded);
 	fprintf(stderr, "Search URL: %s\n", url);
     free(encoded);
 
@@ -99,7 +101,7 @@ int geocodeBgThread(void* query) {
 
     if (downloadAndSave(url, QUERY_TMP_FILENAME)) {
         if (g_file_get_contents(QUERY_TMP_FILENAME, &response, &length, NULL)) {
-        	options.geocodingProvider.parseResponse(response);
+        	options.geocodingProvider -> parseResponse(response);
 
             g_free(response);
             remove(QUERY_TMP_FILENAME);
