@@ -18,7 +18,6 @@
 #include "google_geocoder.h"
 
 #define GOOGLE_QUERY_URL "http://maps.googleapis.com/maps/api/geocode/json?address=%s&sensor=true"
-
 #define KEY_RESULTS "results"
 #define KEY_FORMATTED_ADDRESS "formatted_address"
 #define KEY_GEOMETRY "geometry"
@@ -56,6 +55,10 @@ void google_parse_response(char* response) {
 	size_t length;
 	int i;
 	json_object * jobj = json_tokener_parse(response);
+	if (is_error(jobj)) {
+		searchResultsStatus = PARSE_ERROR;
+		return;
+	}
 	json_object * json_results = json_object_object_get(jobj, KEY_RESULTS);
 	if (json_results != NULL) {
 		length = json_object_array_length(json_results);
@@ -63,7 +66,8 @@ void google_parse_response(char* response) {
 			for (i = 0; i < length; i++) {
 				json_object * json_result = json_object_array_get_idx(json_results, i);
 				GeoCodeResult * result = calloc(1, sizeof(GeoCodeResult));
-				result -> name = json_object_get_string(json_object_object_get(json_result, KEY_FORMATTED_ADDRESS));
+				result -> name = calloc(300, sizeof(char));
+				ascifyAndStripTags((char*) json_object_get_string(json_object_object_get(json_result, KEY_FORMATTED_ADDRESS)), result -> name);
 				json_object *geometry = json_object_object_get(json_result, KEY_GEOMETRY);
 
 				readGoogleWorldCoordinate(json_object_object_get(geometry, KEY_LOCATION), &result -> centroid);
