@@ -24,7 +24,7 @@
 #define GOOGLE_QUERY_URL "http://maps.googleapis.com/maps/api/geocode/json?address=%s&sensor=true"
 
 char* google_maps_local_prepare_url(char* encodedQuery);
-char* google_maps_prepare_url(char* encodedQuery);
+char* google_maps_prepare_url(void* encodedQuery);
 void google_maps_parse_response(char* response);
 
 #endif /* GOOGLE_LOCAL_GEOCODER_H_ */
