@@ -45,10 +45,16 @@ void processMouse() {
 	if (mouse.button == 0) {
 		mouse.xdelta = 0;
 		mouse.ydelta = 0;
+		mouse.draggedDistanceX = 0;
+		mouse.draggedDistanceY = 0;
 	}
 
-	if (mouse.button == 1 && mouse.oldButton == 0) {
+	if (mouse.button != 0 && mouse.oldButton == 0) {
 		mouse.pressed = nowMillis;
+		mouse.clickedx = mouse.x;
+		mouse.clickedy = mouse.y;
+		mouse.draggedDistanceX = 0;
+		mouse.draggedDistanceY = 0;
 	}
 
 	if (mouse.oldButton == 1 && mouse.button == 1) {
@@ -56,6 +62,16 @@ void processMouse() {
 		mouse.ydelta = mouse.y - mouse.oldy;
 		if (mouse.xdelta != 0 || mouse.ydelta != 0) {
 			mouse.moved = nowMillis;
+
+			int draggedDistanceX = abs(mouse.x - mouse.clickedx);
+			int draggedDistanceY = abs(mouse.y - mouse.clickedy);
+
+			if (draggedDistanceX > mouse.draggedDistanceX) {
+				mouse.draggedDistanceX = draggedDistanceX;
+			}
+			if (draggedDistanceY > mouse.draggedDistanceY) {
+				mouse.draggedDistanceY = draggedDistanceY;
+			}
 		}
 	}
 	tileEngineProcessMouse(processUiMouse());
@@ -96,14 +112,46 @@ void processTextInput(char* text) {
 		}
 		computeCursorScreen();
 	}
-//	fprintf(stderr, "textInput: %d %s '%s'\n", strlen(searchBar.input), text, searchBar.input);
+	//	fprintf(stderr, "textInput: %d %s '%s'\n", strlen(searchBar.input), text, searchBar.input);
 }
-void tmpAddMarker();
+
+BackgroundQueryProvider* getNextProvider(GList* allProviders, BackgroundQueryProvider* currentProvider, int reverseSearch) {
+	GList* provider = allProviders;
+
+	while (provider != NULL) {
+		BackgroundQueryProvider *backgroundQueryProvider = provider -> data;
+		if (backgroundQueryProvider == currentProvider) {
+			if (reverseSearch) {
+				if (provider -> prev != NULL) {
+					return provider -> prev -> data;
+				} else {
+					return g_list_last(allProviders) -> data;
+				}
+			} else {
+				if (provider -> next != NULL) {
+					return provider -> next -> data;
+				} else {
+					return allProviders -> data;
+				}
+			}
+			break;
+		}
+		provider = provider -> next;
+	}
+}
+
+void addSearchMarkerAtCanvasCenter();
 void processKeyDown(SDL_KeyboardEvent key) {
+	freePopupMenu();
+
 	if (canvas.searchBarActive == 0) {
 		switch (key.keysym.sym) {
 		case SDLK_b:
-			buildings3dtest();
+			if (key.keysym.mod & KMOD_LSHIFT) {
+				clearBuildings();
+			} else {
+				buildings3dtest();
+			}
 			break;
 		case SDLK_g:
 			options.showGrid = 1 - options.showGrid;
@@ -121,6 +169,14 @@ void processKeyDown(SDL_KeyboardEvent key) {
 			options.accelerometerEnabled = 1 - options.accelerometerEnabled;
 			accelerometer.performCalibration = TRUE;
 			break;
+		case SDLK_z:
+			options.offlineMode = 1 - options.offlineMode;
+			if (options.offlineMode) {
+				addConsoleLine("Off-line mode activated", 0, 1, 0);
+			} else {
+				addConsoleLine("Off-line mode deactivated", 0, 1, 0);
+			}
+			break;
 		case SDLK_m:
 			options.mipmapToggle = 1 - options.mipmapToggle;
 			recreateTiles();
@@ -129,7 +185,14 @@ void processKeyDown(SDL_KeyboardEvent key) {
 			tileEngineChangeOrientation(1 - options.orientation);
 			break;
 		case SDLK_o:
-			tmpAddMarker();
+			addSearchMarkerAtCanvasCenter();
+			break;
+		case SDLK_u:
+			if (key.keysym.mod & KMOD_LSHIFT) {
+				tileEngineFreeRoute();
+			} else {
+				tmpQueryRoute();
+			}
 			break;
 		case SDLK_v:
 			addConsoleLine("CloudGPS ver. " CLOUDGPS_VERION, 0, 1, 0);
@@ -151,32 +214,15 @@ void processKeyDown(SDL_KeyboardEvent key) {
 			}
 			break;
 		case SDLK_s: {
-			int i, oldIdx, newIdx;
-
-	    	for (i = 0; i < geocodingProviders -> len; i++) {
-	    		if (g_str_equal(g_array_index(geocodingProviders, GeocodingProvider, i).name, options.geocodingProvider.name)) {
-	    			oldIdx = i;
-	    			break;
-	    		}
-	    	}
-
-			if (key.keysym.mod & KMOD_LSHIFT) {
-				if (oldIdx > 0) {
-					newIdx = oldIdx - 1;
-				} else {
-					newIdx = geocodingProviders -> len - 1;
-				}
-			} else {
-				newIdx = oldIdx + 1;
-				if (newIdx >= geocodingProviders -> len) {
-					newIdx = 0;
-				}
-			}
-			options.geocodingProvider = g_array_index(geocodingProviders, GeocodingProvider, newIdx);
-			sprintf(strbuf, "New search provider: %s\n", options.geocodingProvider.name);
+			options.geocodingProvider = getNextProvider(geocodingProviders, options.geocodingProvider, key.keysym.mod & KMOD_LSHIFT);
+			sprintf(strbuf, "New search provider: %s\n", options.geocodingProvider -> name);
 			addConsoleLine(strbuf, 0, 1.0, 0);
 		}
 			break;
+		case SDLK_d: {
+			canvas.routeFlybyMode = 1;
+		}
+			break;
 		case SDLK_n: {
 			GList *provider = tileProviders;
 
@@ -206,8 +252,14 @@ void processKeyDown(SDL_KeyboardEvent key) {
 			recreateTiles();
 		}
 			break;
+		case SDLK_r: {
+			options.routingProvider = getNextProvider(routingProviders, options.routingProvider, key.keysym.mod & KMOD_LSHIFT);
+			sprintf(strbuf, "New routing provider: %s\n", options.routingProvider -> name);
+			addConsoleLine(strbuf, 0, 1.0, 0);
+		}
+			break;
 		default:
-//			fprintf(stderr, "unhandled key down: key name %s\n", SDL_GetKeyName(key.keysym.sym));
+			//			fprintf(stderr, "unhandled key down: key name %s\n", SDL_GetKeyName(key.keysym.sym));
 			break;
 		}
 	} else {
@@ -274,6 +326,7 @@ void processKeyboard() {
 				canvas.attractedToRotZ = 0;
 				canvas.drotz += .2;
 			} else {
+				canvas.routeFlybyMode = 0;
 				canvas.followingMypos = 0;
 				canvas.attractionToPoint = 0;
 				canvas.friction = 0.97;
@@ -286,6 +339,7 @@ void processKeyboard() {
 				canvas.attractedToRotZ = 0;
 				canvas.drotz -= .2;
 			} else {
+				canvas.routeFlybyMode = 0;
 				canvas.followingMypos = 0;
 				canvas.friction = 0.97;
 
@@ -295,6 +349,7 @@ void processKeyboard() {
 			}
 		}
 		if (keys[SDLK_UP]) {
+			canvas.routeFlybyMode = 0;
 			canvas.followingMypos = 0;
 			canvas.attractionToPoint = 0;
 			canvas.friction = 0.97;
@@ -303,6 +358,7 @@ void processKeyboard() {
 			canvas.dy += 0.2 * cos((canvas.rotz + 90 * options.orientation) * M_PI / 180.0);
 		}
 		if (keys[SDLK_DOWN]) {
+			canvas.routeFlybyMode = 0;
 			canvas.followingMypos = 0;
 			canvas.attractionToPoint = 0;
 			canvas.friction = 0.97;
