@@ -20,11 +20,13 @@
 #include <locale.h>
 
 #include <SDL/SDL.h>
-#include <SDL/SDL_ttf.h>
+//#include <SDL/SDL_ttf.h>
 
 #ifdef N900
 #include <location/location-gps-device.h>
 #include <location/location-gpsd-control.h>
+LocationGPSDevice *device;
+LocationGPSDControl *control;
 #endif
 
 #include "main.h"
@@ -34,21 +36,18 @@
 #include "screen.h"
 #include "tileProviders.c"
 
-#ifdef N900
-LocationGPSDevice *device;
-LocationGPSDControl *control;
-#endif
-
 #include "taskman.c"
 #include "tileengine.h"
 #include "glcanvas.c"
 #include "accelerometer.c"
 #include "geocoding/geocoder.h"
+#include "routing/router.h"
 #include "buildings3d.c"
 #include "input.c"
 #include "config.c"
 #include "location.c"
 #include "battery.c"
+#include "navigation/navigation.h"
 
 Canvas canvas;
 
@@ -101,23 +100,26 @@ int main(int argc, char **argv) {
 	int isActive = TRUE;
 
 	initGL();
-#ifdef N900
+#if defined(N900) || defined(N950)
 	SDL_ShowCursor(0);
 #else
 	SDL_ShowCursor(1);
 #endif
 
 
-	if (TTF_Init() == -1) {
-		fprintf(stderr, "TTF_Init: %s\n", TTF_GetError());
-		exit(2);
-	}
+//	if (TTF_Init() == -1) {
+//		fprintf(stderr, "TTF_Init: %s\n", TTF_GetError());
+//		exit(2);
+//	}
 
 	loadUI();
 	loadConfig();
 	initInput();
 
 	initTileEngine();
+	if (options.loadLastRoute) {
+		loadRoute(NULL);
+	}
 	initLocation();
 	initBattery();
 	curl_global_init(CURL_GLOBAL_ALL);
@@ -162,6 +164,7 @@ int main(int argc, char **argv) {
 		processGeocoding();
 		processAccelerometer();
 		tileEngineUpdateCoordinates();
+		processRouting();
 		tileEngineUpdateMarkers();
 		setupGlScene();
 		processMouse();
@@ -169,13 +172,14 @@ int main(int argc, char **argv) {
 		tileEngine_computeTilesVisibility();
 		tileEngine_handleZoomTiles();
 		updateUi();
-		deleteOldLines();
+		deleteOldConsoleLines();
 		updateSyncWaitTiles();
-
+		updateBusyValue();
+		updateNavigation();
 		// TODO detect screen lock
 		if (!isActive) {
 			// when our window is inactive save system resources by limiting framerate
-#ifdef N900
+#if defined(N900) || defined(N950)
 			SDL_Delay(100);
 #endif
 		}
@@ -195,7 +199,7 @@ int main(int argc, char **argv) {
 		prevent_screen_dimming();
 	}
 	saveConfig();
-	TTF_Quit();
+//	TTF_Quit();
 	SDL_Quit();
 	return 0;
 }
