@@ -23,6 +23,9 @@ size_t writeToFileStream(void *ptr, size_t size, size_t nmemb, void *stream) {
 };
 
 int downloadAndStream(char *url, void* stream, void *writeToStream) {
+	if (options.offlineMode) {
+		return FALSE;
+	}
     CURL *curl = curl_easy_init();
     CURLcode res;
     long http_code;
@@ -31,6 +34,9 @@ int downloadAndStream(char *url, void* stream, void *writeToStream) {
         return FALSE;
     }
     curl_easy_setopt(curl, CURLOPT_URL, url);
+    if (options.forceIPv4) {
+		curl_easy_setopt(curl, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);
+    }
     if (options.referer != NULL) {
         curl_easy_setopt(curl, CURLOPT_REFERER, options.referer);
     }
