@@ -20,23 +20,18 @@
 double avgR = 1, avgG = 1, avgB = 1, clearR = 1, clearG = 1, clearB = 1;
 
 /* function to load in bitmap as SDL surface ready to be used in OpenGL */
-SDL_Surface* loadTextureFromFile(char *filename, t_tile* tile,
-		SDL_Surface** mask) {
+SDL_Surface* loadTextureFromFile(char *filename, t_tile* tile, SDL_Surface** mask) {
 	SDL_Surface *surface;
 	SDL_Surface *surfaceRGB;
 	SDL_Rect rect;
 
-	//    fprintf(stderr, "IMG_Load %s\n", filename);
 
 	if ((surface = IMG_Load(filename))) {
-		//        fprintf(stderr, "IMG_Load succeeded\n");
 
 		// convert loaded image to RGBA image and then create opengl texture
 
 		if (mask != NULL) {
-			(*mask) = SDL_CreateRGBSurface(SDL_SWSURFACE, surface->w,
-					surface->h, 16, 0x00000F000, 0x00000F00, 0x000000F0,
-					0x000000F);
+			(*mask) = SDL_CreateRGBSurface(SDL_SWSURFACE, surface->w, surface->h, 16, 0x00000F000, 0x00000F00, 0x000000F0, 0x000000F);
 			if ((*mask) == NULL) {
 				fprintf(stderr, "Unable to create mask surface.\n");
 				exit(-1);
@@ -52,8 +47,7 @@ SDL_Surface* loadTextureFromFile(char *filename, t_tile* tile,
 			}
 		}
 
-		surfaceRGB = SDL_CreateRGBSurface(SDL_SWSURFACE, surface->w,
-				surface->h, 16, 0x00000F000, 0x00000F00, 0x000000F0, 0x000000F);
+		surfaceRGB = SDL_CreateRGBSurface(SDL_SWSURFACE, surface->w, surface->h, 16, 0x00000F000, 0x00000F00, 0x000000F0, 0x000000F);
 
 		if (surfaceRGB) {
 			rect.x = 0;
@@ -92,14 +86,11 @@ SDL_Surface* loadTextureFromFile(char *filename, t_tile* tile,
 			fprintf(stderr, "createRGBsurface failed \n");
 		}
 	} else {
-		if (tile) {
+		if (tile == NULL) {
 			// TODO if file exists error message should be written
-			//			char* errorMessage;
-			//			errorMessage = (char*) SDL_GetError();
-			//			fprintf(
-			//					stderr,
-			//					"loading from file '%s' failed, file will be removed, error: %s\n",
-			//					filename, errorMessage);
+			char* errorMessage;
+			errorMessage = (char*) SDL_GetError();
+			fprintf(stderr, "loading from file '%s' failed, error: %s\n", filename, errorMessage);
 			//			remove(filename);
 		}
 	}
@@ -113,7 +104,7 @@ volatile long allocatedTexturesSize = 0;
 
 void deleteTexture(GLuint texture) {
 	int textureSize = (int) g_hash_table_lookup(textureSizeHashTable, texture);
-//	fprintf(stderr, "delete texture %d, size = %d, allocated total = %ld\n", texture, textureSize, allocatedTexturesSize);
+	//	fprintf(stderr, "delete texture %d, size = %d, allocated total = %ld\n", texture, textureSize, allocatedTexturesSize);
 	glDeleteTextures(1, &texture);
 	allocatedTexturesSize -= textureSize;
 }
@@ -132,23 +123,21 @@ GLuint createTexture(const GLushort * pixels4444, int w, int h, int isTile) {
 	glGenTextures(1, &texture);
 
 	if (isTile) {
-		g_hash_table_insert(textureSizeHashTable, texture, w*h*2);
-		allocatedTexturesSize += w*h*2;
-//		fprintf(stderr, "create texture %d, size = %d, allocated total = %ld\n", texture, w*h*2, allocatedTexturesSize);
+		g_hash_table_insert(textureSizeHashTable, texture, w * h * 2);
+		allocatedTexturesSize += w * h * 2;
+		//		fprintf(stderr, "create texture %d, size = %d, allocated total = %ld\n", texture, w*h*2, allocatedTexturesSize);
 	}
 
 	glBindTexture(GL_TEXTURE_2D, texture);
 
 	glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_TRUE);
 	if (isTile && options.mipmapToggle) {
-		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
-				GL_LINEAR_MIPMAP_NEAREST);
+		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
 	} else {
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 	}
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
-	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA,
-			GL_UNSIGNED_SHORT_4_4_4_4, pixels4444);
+	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_SHORT_4_4_4_4, pixels4444);
 
 	return texture;
 }
@@ -167,11 +156,9 @@ void updateSyncWaitTiles() {
 	for (i = 0; i < TILES_X; i++) {
 		for (j = 0; j < TILES_Y; j++) {
 			if (tiles[i][j][currentTilesIdx] != NULL) {
-				if (tiles[i][j][currentTilesIdx] -> state == STATE_LOADING
-						&& tiles[i][j][currentTilesIdx] -> visible) {
+				if (tiles[i][j][currentTilesIdx] -> state == STATE_LOADING && tiles[i][j][currentTilesIdx] -> visible) {
 					loadingCount++;
-				} else if (tiles[i][j][currentTilesIdx] -> state
-						== STATE_LOADED_SYNC_WAIT) {
+				} else if (tiles[i][j][currentTilesIdx] -> state == STATE_LOADED_SYNC_WAIT) {
 					syncWaitCount++;
 				}
 			}
@@ -182,16 +169,13 @@ void updateSyncWaitTiles() {
 		lastTileSync = nowMillis;
 	}
 
-	if ((lastTileSync != 0 && nowMillis - lastTileSync > 1000 && syncWaitCount
-			> 0)) {
+	if ((lastTileSync != 0 && nowMillis - lastTileSync > 1000 && syncWaitCount > 0)) {
 		for (i = 0; i < TILES_X; i++) {
 			for (j = 0; j < TILES_Y; j++) {
 				if (tiles[i][j][currentTilesIdx] != NULL) {
-					if (tiles[i][j][currentTilesIdx] -> state
-							== STATE_LOADED_SYNC_WAIT) {
+					if (tiles[i][j][currentTilesIdx] -> state == STATE_LOADED_SYNC_WAIT) {
 						tiles[i][j][currentTilesIdx]->state = STATE_LOADED;
-						tiles[i][j][currentTilesIdx]->stateChangeTime
-								= nowMillis;
+						tiles[i][j][currentTilesIdx]->stateChangeTime = nowMillis;
 						//fprintf(stderr, "%ld - synced tile %d, %d\n", nowMillis, i, j);
 					}
 				}
