@@ -26,8 +26,9 @@ GList* uiElems = NULL;
 
 UiElement* pressedUiElem = NULL;
 
-UiElement *crosshair, *zoomIn, *zoomOut, *position, *gotomypos, *view2d, *view3d, *compass, *search, *zoomKnot, *closeApp, *tasks;
-extern Texture searchMarkTexture, searchMarkMask;
+UiElement *crosshair, *zoomIn, *zoomOut, *position, *gotomypos, *view2d, *view3d,
+          *compass, *search, *zoomKnot, *closeApp, *tasks, *busy;
+extern Texture searchMarkTexture, searchMarkMask, road;
 
 // those need updating on portrait/landscape change.
 GList* volatileUiElems = NULL;
@@ -36,23 +37,29 @@ void loadTextureAndMask(char* filename, Texture* texture, Texture* mask) {
 	SDL_Surface* textureSurface = NULL;
 	SDL_Surface* maskSurface = NULL;
 	//    fprintf(stderr, "loading texture from %s\n", filename);
-	textureSurface = loadTextureFromFile(filename, NULL, &maskSurface);
+	textureSurface = loadTextureFromFile(filename, NULL, mask != NULL ? &maskSurface : NULL);
 	//    fprintf(stderr, "loadTextureFromFile returned: %p\n", textureRGB);
 	if (textureSurface != NULL) {
 		texture -> size = textureSurface -> w;
 		texture -> name = createTexture(textureSurface -> pixels, texture->size, texture-> size, FALSE);
 
-		mask -> size = maskSurface -> w;
-		mask -> name = createTexture(maskSurface -> pixels, mask->size, mask-> size, FALSE);
+		if (mask != NULL) {
+			mask -> size = maskSurface -> w;
+			mask -> name = createTexture(maskSurface -> pixels, mask->size, mask-> size, FALSE);
+			SDL_FreeSurface(maskSurface);
+		}
 
 		SDL_FreeSurface(textureSurface);
-		SDL_FreeSurface(maskSurface);
 	} else {
 		fprintf(stderr, "loadTexture %s failed\n", filename);
 		exit(-1);
 	}
 }
 
+void loadTexture(char* filename, Texture* texture) {
+	loadTextureAndMask(filename, texture, NULL);
+}
+
 UiElement* createUiElement(char* texture, int landscapex, int landscapey, int portraitx, int portraity, void(*handlePressed)(), void(*handleDragged)(), int addToList) {
 	UiElement* element = calloc(1, sizeof(UiElement));
 	if (element == NULL) {
@@ -95,6 +102,30 @@ void destroyUiElement(UiElement* elem) {
 	fprintf(stderr, "STUB - destroyUiElement(UiElement* elem)\n");
 }
 
+void rotate2d(GLfloat* x, GLfloat* y, double rotation);
+void prepareQuadStripForUiElement(UiElement* elem) {
+	// Dirty hack to make position always the same size on map.
+	if (elem == position) {
+		setQuadStripSize(elem->mask.size / canvas.scale, elem->mask.size / canvas.scale, quadStripVertices);
+	} else if (elem == busy) {
+		setQuadStripSize(elem->mask.size, elem->mask.size, quadStripVertices);
+		int i;
+		for (i = 0; i < 4; i++) {
+			quadStripVertices[i * 3] -= elem->mask.size / 2;
+			quadStripVertices[i * 3 + 1] -= elem->mask.size / 2;
+			rotate2d(&quadStripVertices[i * 3], &quadStripVertices[i * 3 + 1], nowMillis / 3.0);
+		}
+	} else {
+		setQuadStripSize(elem->mask.size, elem->mask.size, quadStripVertices);
+	}
+}
+
+void afterUiElementDrawn(UiElement* elem) {
+	if (elem == busy) {
+		resetQuadStripVertices();
+	}
+}
+
 void drawUiElement(UiElement* elem) {
 
 	glTranslatef(elem -> x, elem -> y, 0);
@@ -109,12 +140,7 @@ void drawUiElement(UiElement* elem) {
 	glColor4f(1.0, 1.0, 1.0, elem->color.a);
 	glBindTexture(GL_TEXTURE_2D, elem->mask.name);
 
-	// Dirty hack to make position always the same size on map.
-	if (elem == position) {
-		setQuadStripSize(elem->mask.size / canvas.scale, elem->mask.size / canvas.scale, quadStripVertices);
-	} else {
-		setQuadStripSize(elem->mask.size, elem->mask.size, quadStripVertices);
-	}
+	prepareQuadStripForUiElement(elem);
 
 	glVertexPointer(3, GL_FLOAT, 0, quadStripVertices);
 	glTexCoordPointer(2, GL_FLOAT, 0, elem->texCoords);
@@ -146,6 +172,8 @@ void drawUiElement(UiElement* elem) {
 	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
 
 	glTranslatef(-elem -> x, -elem -> y, 0);
+
+	afterUiElementDrawn(elem);
 }
 void setQuitFlag();
 
@@ -167,14 +195,20 @@ void loadUI() {
 	zoomKnot -> status = UI_HIDDEN;
 
 	tasks = createUiElement("/opt/cloudgps/res/tasks.png", 16, 0, 0, 0, &invokeTaskManager, NULL, TRUE);
-
 	tasks -> status = UI_HIDDEN;
 
 	closeApp = createUiElement("/opt/cloudgps/res/close.png", SCREEN_WIDTH - 64, 32, 32, 32, &setQuitFlag, NULL, TRUE);
-
 	closeApp -> status = UI_HIDDEN;
 
+	busy = createUiElement("/opt/cloudgps/res/busy.png", SCREEN_WIDTH - 96, 32, 32, 32, NULL, NULL, TRUE);
+	busy -> status = UI_HIDDEN;
+
 	loadTextureAndMask("/opt/cloudgps/res/searchmark.png", &searchMarkTexture, &searchMarkMask);
+	loadTextureAndMask("/opt/cloudgps/res/route_start.png", &routeStartTexture, &routeStartMask);
+	loadTextureAndMask("/opt/cloudgps/res/route_end.png", &routeEndTexture, &routeEndMask);
+	loadTexture("/opt/cloudgps/res/shadow.png", &shadow);
+	loadTexture("/opt/cloudgps/res/drag_crosshair.png", &dragCrosshair);
+	loadTexture("/opt/cloudgps/res/road.png", &road);
 
 	volatileUiElems = g_list_append(volatileUiElems, zoomOut);
 	volatileUiElems = g_list_append(volatileUiElems, view2d);
@@ -560,12 +594,131 @@ void drawConsoleLine(ConsoleLine* line) {
 	drawString(line -> text, line -> r, line -> g, line -> b, TRUE);
 }
 
+
+void drawNavigationInstruction(Orientation orientation) {
+	if (canvas.navigationStatus.currentDirection == NULL) {
+		return;
+	}
+	int meters = canvas.navigationStatus.roundedDistanceToNextDirection;
+	if (meters >= 100000) {
+		sprintf(strbuf, "In %dkm: %s", (meters / 1000), canvas.navigationStatus.currentDirection -> text);
+	} else if (meters >= 1000) {
+		sprintf(strbuf, "In %.2fkm: %s", (meters / 1000.0), canvas.navigationStatus.currentDirection -> text);
+	} else {
+		sprintf(strbuf, "In %dm: %s", meters, canvas.navigationStatus.currentDirection -> text);
+	}
+	int textWidth = stringWidth(strbuf);
+
+	meters = canvas.navigationStatus.distanceToTarget;
+	if (meters >= 10000) {
+		sprintf(strbuf2, "Distance to destination %dkm", meters / 1000);
+	} else if (meters >= 1000) {
+		sprintf(strbuf2, "Distance to destination %.2fkm", meters / 1000.0);
+	} else {
+		sprintf(strbuf2, "Distance to destination %d%s", meters, "m");
+	}
+	int textWidth2 = stringWidth(strbuf);
+
+	if (textWidth2 > textWidth) {
+		textWidth = textWidth2;
+	}
+	glDisable(GL_TEXTURE_2D);
+	setBoxSize(textWidth + 40, 40, boxVertices);
+	glPushMatrix();
+	if (orientation == LANDSCAPE) {
+		glTranslatef(SCREEN_WIDTH / 2 - textWidth / 2 - 100, 40, 0);
+	} else {
+		glTranslatef(SCREEN_HEIGHT / 2 - textWidth / 2 - 30, 55, 0);
+	}
+
+	glColor4f(1, 1, 1, 1);
+	glVertexPointer(3, GL_FLOAT, 0, boxVertices);
+	glDrawArrays(GL_LINE_LOOP, 0, 4);
+
+	glBlendFunc(GL_DST_COLOR, GL_ZERO);
+	glColor4f(.4, .4, .4, .7);
+
+	if (orientation == LANDSCAPE) {
+		glTranslatef(1, 0, 0);
+	} else {
+		glTranslatef(1, 1, 0);
+	}
+	setQuadStripSize(textWidth + 39, 40, quadStripVertices);
+
+	glVertexPointer(3, GL_FLOAT, 0, quadStripVertices);
+	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
+
+	glEnable(GL_TEXTURE_2D);
+	glTranslatef(20, 0, 0);
+	drawString(strbuf, 1, 1, 1, FALSE);
+	glTranslatef(0, 20, 0);
+	drawString(strbuf2, 1, 1, 1, FALSE);
+	glPopMatrix();
+}
+
+void drawScale(Orientation orientation) {
+	double groundResolution = getGroundResolution(&canvas.center, canvas.zoom) / canvas.scale;
+	double exponent = log10(groundResolution);
+	double nextPowerOfTen = pow(10, floor(exponent) + 1);
+	double norm = groundResolution / nextPowerOfTen;
+	double scaleWidth;
+
+	if (norm < 0.2) {
+		scaleWidth = 20;
+	} else if (norm < 0.5) {
+		scaleWidth = 50;
+	} else {
+		scaleWidth = 100;
+	}
+
+	int scale = (int) scaleWidth * (int) nextPowerOfTen;
+
+	glDisable(GL_TEXTURE_2D);
+	scaleWidth = scaleWidth / norm;
+	setBoxSize((int) scaleWidth, 14, boxVertices);
+	glPushMatrix();
+	if (orientation == LANDSCAPE) {
+		glTranslatef(-80, 50, 0);
+	} else {
+		glTranslatef(13, 65, 0);
+	}
+
+	glColor4f(1, 1, 1, 1);
+	glVertexPointer(3, GL_FLOAT, 0, boxVertices);
+	glDrawArrays(GL_LINE_LOOP, 0, 4);
+
+	glBlendFunc(GL_DST_COLOR, GL_ZERO);
+	glColor4f(.4, .4, .4, .7);
+
+	if (orientation == LANDSCAPE) {
+		glTranslatef(1, 0, 0);
+	} else {
+		glTranslatef(1, 1, 0);
+	}
+	setQuadStripSize((int) scaleWidth - 1, 13, quadStripVertices);
+
+	glVertexPointer(3, GL_FLOAT, 0, quadStripVertices);
+	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
+
+	if (scale == 0) {
+		sprintf(strbuf, "%2.3f m", (int)scaleWidth * nextPowerOfTen);
+	} else if (scale >= 10000) {
+		sprintf(strbuf, "%d km", scale / 1000);
+	} else {
+		sprintf(strbuf, "%d m", scale);
+	}
+	glEnable(GL_TEXTURE_2D);
+	glTranslatef(scaleWidth/2 - stringWidth(strbuf)/2, -2, 0);
+	drawString(strbuf, 1, 1, 1, FALSE);
+	glPopMatrix();
+}
+
 void drawStatusBar(Orientation orientation) {
 	if (canvas.orientationTransition > 0.001 && canvas.orientationTransition < .999) {
 		if (orientation == LANDSCAPE) {
-			glTranslatef(-SCREEN_WIDTH * canvas.orientationTransition, 0, 0);
+			glTranslatef((int) (-SCREEN_WIDTH * canvas.orientationTransition), 0, 0);
 		} else {
-			glTranslatef(0, SCREEN_HEIGHT * canvas.orientationTransition, 0);
+			glTranslatef(0, (int) (SCREEN_HEIGHT * canvas.orientationTransition), 0);
 		}
 	} else {
 		if (orientation != options.orientation) {
@@ -609,6 +762,18 @@ void drawStatusBar(Orientation orientation) {
 		closeApp -> x = SCREEN_WIDTH - 80;
 		closeApp -> y = 0;
 		drawUiElement(closeApp);
+
+		if (canvas.busy) {
+			busy -> x = SCREEN_WIDTH - 90;
+			busy -> y = 30;
+			drawUiElement(busy);
+		}
+	} else {
+		if (canvas.busy) {
+			busy -> x = SCREEN_HEIGHT - 20;
+			busy -> y = 30;
+			drawUiElement(busy);
+		}
 	}
 
 	glTranslatef(10 + (orientation == LANDSCAPE ? 90 : 0), 5 + (orientation == LANDSCAPE ? 5 : 0), 0);
@@ -621,6 +786,8 @@ void drawStatusBar(Orientation orientation) {
 	}
 
 	glTranslatef(0, 20, 0);
+	drawNavigationInstruction(orientation);
+	drawScale(orientation);
 #ifdef N900
 	switch (device -> fix -> mode) {
 		case LOCATION_GPS_DEVICE_MODE_2D:
@@ -733,8 +900,8 @@ int processUiMouse() {
 					pressedUiElem = e;
 					if (pressedUiElem -> handlePressed != NULL) {
 						pressedUiElem -> handlePressed();
+						result = TRUE;
 					}
-					result = TRUE;
 					break;
 				}
 
